package net.shadowfacts.phycon.network.block.redstone

import net.minecraft.block.BlockState
import net.minecraft.nbt.CompoundTag
import net.shadowfacts.phycon.api.packet.Packet
import net.shadowfacts.phycon.api.util.IPAddress
import net.shadowfacts.phycon.init.PhyBlockEntities
import net.shadowfacts.phycon.network.DeviceBlockEntity
import net.shadowfacts.phycon.network.packet.RemoteActivationPacket
import net.shadowfacts.phycon.util.RedstoneMode

/**
 * @author shadowfacts
 */
class RedstoneControllerBlockEntity: DeviceBlockEntity(PhyBlockEntities.REDSTONE_CONTROLLER) {

	var managedDevices = Array<IPAddress?>(5) { null }
	var redstoneMode = RedstoneMode.HIGH

	private var redstonePowered = false

	override fun handle(packet: Packet) {
	}

	fun redstoneStateChanged() {
		val oldPowered = redstonePowered
		redstonePowered = cachedState[RedstoneControllerBlock.LIT]

		val mode: RemoteActivationPacket.Mode? = when (redstoneMode) {
			RedstoneMode.TOGGLE -> if (oldPowered != redstonePowered) RemoteActivationPacket.Mode.SINGLE else null
			RedstoneMode.RISING_EDGE -> if (!oldPowered && redstonePowered) RemoteActivationPacket.Mode.SINGLE else null
			RedstoneMode.FALLING_EDGE -> if (oldPowered && !redstonePowered) RemoteActivationPacket.Mode.SINGLE else null
			RedstoneMode.HIGH -> if (redstonePowered) RemoteActivationPacket.Mode.ENABLE else RemoteActivationPacket.Mode.DISABLE
			RedstoneMode.LOW -> if (redstonePowered) RemoteActivationPacket.Mode.DISABLE else RemoteActivationPacket.Mode.ENABLE
		}

		if (mode != null) {
			sendActivatePacket(mode)
		}
	}

	private fun sendActivatePacket(mode: RemoteActivationPacket.Mode) {
		for (ip in managedDevices) {
			if (ip == null) continue
			sendPacket(RemoteActivationPacket(mode, ipAddress, ip))
		}
	}

	override fun toTag(tag: CompoundTag): CompoundTag {
		tag.putIntArray("ManagedDevices", managedDevices.mapNotNull { it?.address })
		tag.putString("RedstoneMode", redstoneMode.name)
		return super.toTag(tag)
	}

	override fun fromTag(state: BlockState, tag: CompoundTag) {
		super.fromTag(state, tag)
		val addresses = tag.getIntArray("ManagedDevices")
		managedDevices = (0..4).map { if (it >= addresses.size) null else IPAddress(addresses[it]) }.toTypedArray()
		redstoneMode = RedstoneMode.valueOf(tag.getString("RedstoneMode"))
	}

	override fun toClientTag(tag: CompoundTag): CompoundTag {
		tag.putIntArray("ManagedDevices", managedDevices.mapNotNull { it?.address })
		tag.putString("RedstoneMode", redstoneMode.name)
		return super.toClientTag(tag)
	}

	override fun fromClientTag(tag: CompoundTag) {
		super.fromClientTag(tag)
		val addresses = tag.getIntArray("ManagedDevices")
		managedDevices = (0..4).map { if (it >= addresses.size) null else IPAddress(addresses[it]) }.toTypedArray()
		redstoneMode = RedstoneMode.valueOf(tag.getString("RedstoneMode"))
	}

}
