package net.shadowfacts.phycon.networking

import net.fabricmc.fabric.api.networking.v1.PacketByteBufs
import net.fabricmc.fabric.api.networking.v1.PacketSender
import net.minecraft.network.Packet
import net.minecraft.network.PacketByteBuf
import net.minecraft.server.MinecraftServer
import net.minecraft.server.network.ServerPlayNetworkHandler
import net.minecraft.server.network.ServerPlayerEntity
import net.minecraft.util.Identifier
import net.minecraft.util.registry.Registry
import net.minecraft.util.registry.RegistryKey
import net.shadowfacts.phycon.PhysicalConnectivity
import net.shadowfacts.phycon.network.DeviceBlockEntity
import net.shadowfacts.phycon.network.component.ActivationController
import net.shadowfacts.phycon.util.ActivationMode

/**
 * @author shadowfacts
 */
object C2SConfigureActivationMode: ServerReceiver {
	override val CHANNEL = Identifier(PhysicalConnectivity.MODID, "configure_activation_mode")

	operator fun <T> invoke(be: T): Packet<*> where T: DeviceBlockEntity, T: ActivationController.ActivatableDevice {
		val buf = PacketByteBufs.create()

		buf.writeIdentifier(be.world!!.registryKey.value)
		buf.writeBlockPos(be.pos)
		buf.writeString(be.controller.activationMode.name)

		return createPacket(buf)
	}

	override fun receive(server: MinecraftServer, player: ServerPlayerEntity, handler: ServerPlayNetworkHandler, buf: PacketByteBuf, responseSender: PacketSender) {
		val dimID = buf.readIdentifier()
		val pos = buf.readBlockPos()
		val mode = ActivationMode.valueOf(buf.readString())

		server.execute {
			// todo: check the player is close enough
			val key = RegistryKey.of(Registry.DIMENSION, dimID)
			val world = server.getWorld(key) ?: return@execute
			val device = world.getBlockEntity(pos) ?: return@execute
			if (device !is ActivationController.ActivatableDevice) return@execute
			device.controller.activationMode = mode
			device.markDirty()
		}
	}
}
